from hatchling.version.core import VersionFile
from hatchling.version.source.plugin.interface import VersionSourceInterface


class RegexSource(VersionSourceInterface):
    PLUGIN_NAME = "regex"

    def get_version_data(self) -> dict:
        relative_path = self.config.get("path", "")
        if not relative_path:
            message = "option `path` must be specified"
            raise ValueError(message)

        if not isinstance(relative_path, str):
            message = "option `path` must be a string"
            raise TypeError(message)

        pattern = self.config.get("pattern", "")
        if not isinstance(pattern, str):
            message = "option `pattern` must be a string"
            raise TypeError(message)

        version_file = VersionFile(self.root, relative_path)
        version = version_file.read(pattern=pattern)

        return {"version": version, "version_file": version_file}

    def set_version(self, version: str, version_data: dict) -> None:  # noqa: PLR6301
        version_data["version_file"].set_version(version)



from __future__ import annotations

from typing import TYPE_CHECKING, TypeVar

if TYPE_CHECKING:
    from hatchling.builders.hooks.plugin.interface import BuildHookInterface
    from hatchling.builders.plugin.interface import BuilderInterface
    from hatchling.metadata.plugin.interface import MetadataHookInterface

    T = TypeVar("T", BuilderInterface, BuildHookInterface, MetadataHookInterface)


def load_plugin_from_script(path: str, script_name: str, plugin_class: type[T], plugin_id: str) -> type[T]:
    from importlib.util import module_from_spec, spec_from_file_location

    spec = spec_from_file_location(script_name, path)
    module = module_from_spec(spec)  # type: ignore[arg-type]
    spec.loader.exec_module(module)  # type: ignore[union-attr]

    plugin_finder = f"get_{plugin_id}"
    names = dir(module)
    if plugin_finder in names:
        return getattr(module, plugin_finder)()

    subclasses = []
    for name in names:
        obj = getattr(module, name)
        if obj is plugin_class:
            continue

        try:
            if issubclass(obj, plugin_class):
                subclasses.append(obj)
        except TypeError:
            continue

    if not subclasses:
        message = f"Unable to find a subclass of `{plugin_class.__name__}` in `{script_name}`: {path}"
        raise ValueError(message)

    if len(subclasses) > 1:
        message = (
            f"Multiple subclasses of `{plugin_class.__name__}` found in `{script_name}`, "
            f"select one by defining a function named `{plugin_finder}`: {path}"
        )
        raise ValueError(message)

    return subclasses[0]



from __future__ import annotations

import argparse
from typing import Any


def version_impl(
    *,
    called_by_app: bool,  # noqa: ARG001
    desired_version: str,
) -> None:
    import os

    from hatchling.bridge.app import Application
    from hatchling.metadata.core import ProjectMetadata
    from hatchling.plugin.manager import PluginManager

    app = Application()

    root = os.getcwd()
    plugin_manager = PluginManager()
    metadata = ProjectMetadata(root, plugin_manager)

    if "version" in metadata.config.get("project", {}):
        if desired_version:
            app.abort("Cannot set version when it is statically defined by the `project.version` field")
        else:
            app.display(metadata.core.version)
            return

    source = metadata.hatch.version.source

    version_data = source.get_version_data()
    original_version = version_data["version"]

    if not desired_version:
        app.display(original_version)
        return

    updated_version = metadata.hatch.version.scheme.update(desired_version, original_version, version_data)
    source.set_version(updated_version, version_data)

    app.display_info(f"Old: {original_version}")
    app.display_info(f"New: {updated_version}")


def version_command(subparsers: argparse._SubParsersAction, defaults: Any) -> None:
    parser = subparsers.add_parser("version")
    parser.add_argument("desired_version", default="", nargs="?", **defaults)
    parser.add_argument("--app", dest="called_by_app", action="store_true", help=argparse.SUPPRESS)
    parser.set_defaults(func=version_impl)
